@@ -86,6 +86,8 @@ bool do_imaginary_j = false;
 
 QalculateDateTime last_version_check_date;
 
+#define VERSION_BEFORE(i1, i2, i3) (version_numbers[0] < i1 || (version_numbers[0] == i1 && (version_numbers[1] < i2 || (version_numbers[1] == i2 && version_numbers[2] < i3))))
+
 static GOptionEntry options[] = {
 	{"file", 'f', 0, G_OPTION_ARG_STRING, NULL, N_("Execute expressions and commands from a file"), N_("FILE")},
 	{"new-instance", 'n', 0, G_OPTION_ARG_NONE, NULL, N_("Start a new instance of the application"), NULL},
@@ -182,7 +184,7 @@ void create_application(GtkApplication *app) {
 	load_preferences();
 
 	mstruct = new MathStructure();
-	displayed_mstruct = new MathStructure();
+	displayed_mstruct = NULL;
 	parsed_mstruct = new MathStructure();
 	parsed_tostruct = new MathStructure();
 	parsed_tostruct->setUndefined();
@@ -207,38 +209,23 @@ void create_application(GtkApplication *app) {
 	while(gtk_events_pending()) gtk_main_iteration();
 	test_border();
 
-	showing_first_time_message = first_time || version_numbers[0] != 4 || version_numbers[1] != 1;
+	showing_first_time_message = first_time;
+
+	if((first_time || VERSION_BEFORE(4, 1, 1)) && QalculateDateTime(2022, 9, 1).isFutureDate()) {
+		gint scalefactor = gtk_widget_get_scale_factor(GTK_WIDGET(gtk_builder_get_object(main_builder, "main_window")));
+		GdkPixbuf *flagbuf = gdk_pixbuf_new_from_resource_at_scale("/qalculate-gtk/flags/UAH.png", 16 * scalefactor, 16 * scalefactor, TRUE, NULL);
+		if(flagbuf) {
+			cairo_surface_t *s = gdk_cairo_surface_create_from_pixbuf(flagbuf, scalefactor, NULL);
+			gtk_image_set_from_surface(GTK_IMAGE(gtk_builder_get_object(main_builder, "message_tooltip_icon")), s);
+			gtk_widget_set_visible(GTK_WIDGET(gtk_builder_get_object(main_builder, "expression_button")), TRUE);
+			gtk_stack_set_visible_child(GTK_STACK(gtk_builder_get_object(main_builder, "expression_button_stack")), GTK_WIDGET(gtk_builder_get_object(main_builder, "message_tooltip_icon")));
+			cairo_surface_destroy(s);
+			g_object_unref(flagbuf);
+		}
+	}
 
 	if(calc_arg.empty() && showing_first_time_message && file_arg.empty()) {
-#if GDK_MAJOR_VERSION > 3 || GDK_MINOR_VERSION >= 22
-		GdkDrawingContext *gdc = gdk_window_begin_draw_frame(gtk_widget_get_window(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview"))), cairo_region_create());
-		cairo_t *cr = gdk_drawing_context_get_cairo_context(gdc);
-#else
-		cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview"))));
-#endif
-		/*PangoLayout *layout = gtk_widget_create_pango_layout(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview")), NULL);
-		GdkRGBA rgba;
-		pango_layout_set_markup(layout, (string("<span size=\"smaller\">") + string(_("Type a mathematical expression above, e.g. \"5 + 2 / 3\",\nand press the enter key.")) + "</span>").c_str(), -1);
-		gtk_style_context_get_color(gtk_widget_get_style_context(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview"))), gtk_widget_get_state_flags(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview"))), &rgba);
-		cairo_move_to(cr, 6, 6);
-		gdk_cairo_set_source_rgba(cr, &rgba);
-		pango_cairo_show_layout(cr, layout);
-		g_object_unref(layout);*/
-		gint h = gtk_widget_get_allocated_height(GTK_WIDGET(gtk_builder_get_object(main_builder, "scrolled_result")));
-		gint w = gtk_widget_get_allocated_width(GTK_WIDGET(gtk_builder_get_object(main_builder, "scrolled_result")));
-		gint uah_h = 16, uah_w = 24;
-		if(h >= 48) {uah_h = 32; uah_w = 48;}
-		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_resource_at_scale("/qalculate-gtk/flags/UAH.png", uah_w, uah_h, TRUE, NULL);
-		cairo_surface_t *s = gdk_cairo_surface_create_from_pixbuf(pixbuf, 1, NULL);
-		cairo_set_source_surface(cr, s, (w - uah_w) / 2, (h - uah_h) / 2);
-		cairo_paint(cr);
-		g_object_unref(pixbuf);
-		cairo_surface_destroy(s);
-#if GDK_MAJOR_VERSION > 3 || GDK_MINOR_VERSION >= 22
-		gdk_window_end_draw_frame(gtk_widget_get_window(GTK_WIDGET(gtk_builder_get_object(main_builder, "resultview"))), gdc);
-#else
-		cairo_destroy(cr);
-#endif
+		gtk_widget_queue_draw(resultview);
 	}
 
 	while(gtk_events_pending()) gtk_main_iteration();
@@ -340,8 +327,9 @@ void create_application(GtkApplication *app) {
 		printops.base = 10;
 		setResult(NULL, false, false, false);
 		printops.base = base;
+	} else if(!showing_first_time_message) {
+		gtk_widget_queue_draw(resultview);
 	}
-	gtk_widget_queue_draw(resultview);
 
 	block_completion();
 	set_custom_buttons();
